import typer
import subprocess

from typing import Optional
from pathlib import Path
from glob import glob

from mdpaper import settings

from datetime import date

app = typer.Typer()

basecmd = "pandoc"

today = date.today()
    
output_vars = {
    "date": today,
    "y": today.year,
    "m": today.month,
    "d": today.day,
}


@app.callback()
def callback():
    """
    Build a document based on a collection of markdown files.
    """

def _build_args(output, fmt):
    
    args = [settings.pandoc, "-s", "-F", "pandoc-crossref"]
    for pandoc_filter in settings.pandoc_filters:
        args.extend(["-F", pandoc_filter])
    if settings.toc:
        args.append("--toc")
        args.extend(["--toc-depth", settings.toc_depth])
    if settings.references:
        args.extend(["--bibliography", settings.references, "--citeproc"])
    
    files_ = []
    for pattern in settings.index:
        for fp_ in glob(pattern):
            fp = Path(fp_)
            assert fp.is_file(), f"{fp_} does not exist"
            files_.append(str(fp.expanduser().resolve()))

    args.extend(files_)

    if output:
        basename = str(output)
        
    elif settings.output:
        basename = settings.output
    
    output_name = (f"{basename}.{fmt}").format(**output_vars)
    args.extend(["-o", output_name])

    return args, output_name

    

@app.command()
def docx(output: Optional[Path] = typer.Option(default=None, resolve_path=True)):
    """
    Build a .docx document.
    """

    args, output_name = _build_args(output, "docx")

    if settings.template_docx:
        args.extend(["--reference-doc", str(settings.template_docx)])

    # Write to output file
    
    if output:
        basename = str(output)
        
    elif settings.output:
        basename = settings.output

    typer.echo(f"Writing file to {output_name}")

    # typer.echo(args)
    subprocess.run(args)


@app.command()
def pdf(output: Optional[Path] = typer.Option(default=None, resolve_path=True)):
    """
    Build a .pdf document.
    """
    
    args, output_name = _build_args(output, "pdf")
    
    args.extend(["--pdf-engine", settings.pdf_engine])

    typer.echo(f"Writing file to {output_name}")

    # typer.echo(args)
    subprocess.run(args)



"""
Add two directives for .docx elements.
"""

import panflute as pf
from enum import Enum

pagebreak = """<w:p><w:r><w:br w:type="page" /></w:r></w:p>"""

sectionbreak = """<w:p><w:pPr><w:sectPr><w:type w:val="nextPage" /></w:sectPr></w:pPr></w:p>"""

toc = r"""
<w:sdt>
<w:sdtPr>
  <w:docPartObj>
    <w:docPartGallery w:val="Table of Contents" />
  </w:docPartObj>
</w:sdtPr>
<w:sdtContent xmlns:w="http://schemas.openxmlformats.org/wordprocessingml/2006/main">
  <w:p>
    <w:pPr>
      <w:pStyle w:val="TOC Heading" />
    </w:pPr>
    <w:r>
      <w:t xml:space="preserve">Table of Contents</w:t>
    </w:r>
  </w:p>
  <w:p>
    <w:r>
      <w:fldChar w:fldCharType="begin" w:dirty="true" />
      <w:instrText xml:space="preserve">TOC \o "1-3" \h \z \u</w:instrText>
      <w:fldChar w:fldCharType="separate" />
      <w:fldChar w:fldCharType="end" />
    </w:r>
  </w:p>
</w:sdtContent>
</w:sdt>
"""

# Directives to add
class Cmd:
    NEWPAGE = r"\newpage"
    TOC = r"\tableofcontents"
    DOCX = "docx"

directives = {
    Cmd.NEWPAGE: pf.RawBlock(pagebreak, format="openxml"),
    Cmd.TOC: pf.RawBlock(toc, format="openxml")
}

class Docxtras:
    """Enable newpage and toc directives for .docx outputs.
    """

    def action(self, elem, doc):
        if (doc.format == Cmd.DOCX) and isinstance(elem, pf.RawBlock):
            if (elem.text == Cmd.NEWPAGE):
                elem = directives[Cmd.NEWPAGE]
            elif (elem.text == Cmd.TOC):
                if (doc.format == "docx"):
                    # para = [pf.Para(pf.Str("TABLE"), pf.Space(), pf.Str("OF"), pf.Space(), pf.Str("CONTENTS"))]
                    # div = pf.Div(*para, attributes={"custom-style": "TOC Header"})
                    elem = directives[Cmd.TOC]
                else:
                    elem = []
        return elem


def main(doc=None):
    dp = Docxtras()
    return pf.run_filter(dp.action, doc=doc)

if __name__ == "__main__":
    main()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mdpaper']

package_data = \
{'': ['*']}

install_requires = \
['dynaconf>=3.1.7,<4.0.0',
 'invoke>=1.6.0,<2.0.0',
 'pandoc-docx-pagebreak>=0.0.2,<0.0.3',
 'typer[all]>=0.4.0,<0.5.0']

entry_points = \
{'console_scripts': ['md-paper = mdpaper.main:app',
                     'pandoc-docxtras = mdpaper.filters:main']}

setup_kwargs = {
    'name': 'mdpaper',
    'version': '0.0.2',
    'description': 'A package for organizing and compiling a thesis type document with markdown.',
    'long_description': None,
    'author': 'Thomas Tu',
    'author_email': 'thomasthetu@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)




from dynaconf import Dynaconf, constants
from pathlib import Path

_settings = Dynaconf(
    envvar_prefix="MDPAPER",
    load_dotenv=True,
    warn_dynaconf_global_settings=True,
    environments=True,
    default_env="mdpaper",
    default_settings_paths=constants.DEFAULT_SETTINGS_FILES,
)

# Get list of index names
index = _settings.get("index", [])

# Reference file
references = _settings.get("references", "references.bib")

# Output file name
output = _settings.get("output")

# Pandoc binary
pandoc = _settings.get("pandoc", "pandoc")

# toc settings
toc = _settings.get("toc", True)
toc_depth = str(_settings.get("toc_depth", 2))

# Additional Filters
pandoc_filters = _settings.get("pandoc_filters", [])

# DOCX Settings
template_docx = _settings.get("template_docx", None)
if template_docx:
    template_docx = Path(template_docx).expanduser().resolve()

# PDF Settings
pdf_engine = _settings.get("pdf_engine", "xelatex")